@@ -10,6 +10,7 @@ import numpy as np
 import pandas as pd
 import skimage.io
 
+from scipy.spatial import procrustes
 from scipy.stats import zscore
 
 from . import warper
@@ -509,3 +510,88 @@ class Modeller():
                 tem_file.write( ' '.join( line_value ) + '\n' )
 
         return None
+    
+    
+    def return_arrays(self, proc_fit_shape = False, as_frame = False):
+        """ Method to return a flattened representation of the shape and colour data, alongside respective measurements. This is useful for manual
+        exploration of the data outside of the Modeller class, for example in PCA or morphometric applications.
+        
+        Shape values are returned as floats, represented as follows: [pt0_X, pt0_Y, ..., ptN_X, ptN_y]
+        Colour values are returned as unsigned 8-bit integers, represented as follows: [pix0_R, pix0_G, pix0_B, ..., pixN_R, pixN_G, pixN_B]
+        
+        Parameters
+        ----------
+        proc_fit_shape : Returns the flattend shape array, but first subjects the shape data to a Procrustes fit, aligning to the average face. Note - this fit returns the coordinates in original scaling, not unit scaling.
+        as_frame : Returns the flattened data as Pandas DataFrames, with meaningful column headers and index. Default False.
+        
+        Returns
+        ----------
+        shape_array : Array or DataFrame representing flattened shape values. Rows are faces, columns landmarks. Size n_obs by n_landmarks * n_dimensions.
+        colour_array : Array or DataFrame representing flattened image pixel values. Rows are faces, columns landmarks. Size n_obs by image height * image width * 3.
+        param_array :  Array or DataFrame representing predictor values entered to Modeller class.
+        """
+        
+        # Create empty arrays for shape, texture, and data
+        # Get number of rows
+        n_obs = self.master_data_frame.shape[0]
+        
+        # Set number of columns for each output
+        n_landmarks = np.prod(self.template_dims)
+        n_pixels = np.prod(self.image_dims) * 3
+        n_params = self.master_data_frame.shape[1]
+        
+        # Prepare empty arrays to store the output, as well as a list for name indices
+        shape_array = np.empty((n_obs, n_landmarks))
+        colour_array = np.empty((n_obs, n_pixels))
+        param_array = np.empty((n_obs, n_params))
+        face_list = []
+        
+        # Populate the arrays
+        for index, (face, values) in enumerate(self.gathered_data.items()):
+            
+            # Get shape attributes
+            shape_array[index,:] = values['shape'].flatten()
+            
+            # Get colour attributes
+            colour_array[index,:] = np.dstack((values['channel_one'], values['channel_two'], values['channel_three'])).flatten()
+            
+            # Get parameters
+            param_array[index,:] = [values[measure] for measure in self.trait_list]
+            
+            # Store ID
+            face_list.append(face)
+        
+        # Convert colour array to unsigned integer, keeping with image values
+        colour_array = colour_array.astype('uint8')
+        
+        # Carry out Procrustes fit if requested
+        if proc_fit_shape:
+            def proc(to_fit):
+                
+                # Reshape to_fit
+                to_fit = to_fit.reshape(self.template_dims)
+                
+                # Call procrustes
+                _, pr_fit, _ = procrustes(self.average_shape, to_fit)
+                
+                # Reverse scaling
+                pr_fit *= np.linalg.norm(to_fit - to_fit.mean(0))
+                pr_fit += to_fit.mean(0)
+                
+                return pr_fit.flatten()
+            
+            shape_array = np.apply_along_axis(proc, 1, shape_array)
+                
+        # Prepare the column names for the dataframes, if chosen
+        if as_frame:
+            shape_ids = (title for pt in range(0, self.template_dims[0]) for title in ['pt{}_X'.format(pt), 'pt{}_Y'.format(pt)])
+            pixel_ids = (title for pt in range(0, np.prod(self.image_dims)) for title in ['px{}_R'.format(pt), 'px{}_G'.format(pt), 'px{}_B'.format(pt)])
+            param_ids = self.trait_list
+            
+            shape_array = pd.DataFrame(shape_array, index=face_list, columns=shape_ids)
+            colour_array = pd.DataFrame(colour_array, index=face_list, columns=pixel_ids)
+            param_array = pd.DataFrame(param_array, index=face_list, columns=param_ids)
+            
+        return shape_array, colour_array, param_array
+        
+        
